@@ -5,5 +5,4 @@
 (setq load-path (cons "." load-path)
       byte-compile-warnings nil
       TeX-lisp-directory "<none>"
-      TeX-auto-global "<none>"
-      TeX-macro-global nil)
+      TeX-auto-global "<none>")
