@@ -1,7 +1,7 @@
 ;;; tex-buf.el --- External commands for AUCTeX.
 
-;; Copyright (C) 1991, 1993, 1996, 2001, 2003, 2004,
-;;   2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+;; Copyright (C) 1991, 1993, 1996, 2001, 2003, 2004, 2005, 2006, 2007,
+;;   2008, 2009 Free Software Foundation, Inc.
 
 ;; Maintainer: auctex-devel@gnu.org
 ;; Keywords: tex, wp
@@ -76,7 +76,7 @@
 
 (defun TeX-save-document (name)
   "Save all files belonging to the current document.
-Return non-nil if document need to be re-TeX'ed."
+Return non-nil if document needs to be re-TeX'ed."
   (interactive (list (TeX-master-file)))
   (if (string-equal name "")
       (setq name (TeX-master-file)))
@@ -366,36 +366,38 @@ without further expansion."
   command)
 
 (defun TeX-check-files (derived originals extensions)
-  "Check that DERIVED is newer than any of the ORIGINALS.
+  "Check if DERIVED is newer than any of the ORIGINALS.
 Try each original with each member of EXTENSIONS, in all directories
-in `TeX-check-path'."
-  (let ((found nil)
-	(regexp (concat "\\`\\("
-			(mapconcat (lambda (dir)
-				     (regexp-quote
-				      (expand-file-name
-				       (file-name-as-directory dir))))
-				   TeX-check-path "\\|")
-			"\\).*\\("
-			(mapconcat 'regexp-quote originals "\\|")
-			"\\)\\.\\("
-			(mapconcat 'regexp-quote extensions "\\|")
-			"\\)\\'"))
-	(buffers (buffer-list)))
+in `TeX-check-path'. Returns true if any of the ORIGINALS with any of the
+EXTENSIONS are newer than DERIVED. Will prompt to save the buffer of any
+ORIGINALS which are modified but not saved yet."
+  (let (existingoriginals
+        found
+	(extensions (TeX-delete-duplicate-strings extensions))
+        (buffers (buffer-list)))
+    (dolist (path (mapcar (lambda (dir)
+			    (expand-file-name (file-name-as-directory dir)))
+			  TeX-check-path))
+      (dolist (orig originals)
+	(dolist (ext extensions)
+	  (let ((filepath (concat path orig "." ext)))
+	    (if (file-exists-p filepath)
+                (setq existingoriginals (cons filepath existingoriginals)))))))
     (while buffers
       (let* ((buffer (car buffers))
-	     (name (buffer-file-name buffer)))
-	(setq buffers (cdr buffers))
-	(if (and name (string-match regexp name))
-	    (progn
-	      (and (buffer-modified-p buffer)
-		   (or (not TeX-save-query)
-		       (y-or-n-p (concat "Save file "
-					 (buffer-file-name buffer)
-					 "? ")))
-		   (save-excursion (set-buffer buffer) (save-buffer)))
-	      (if (file-newer-than-file-p name derived)
-		  (setq found t))))))
+             (name (buffer-file-name buffer)))
+        (setq buffers (cdr buffers))
+        (if (and name (member name existingoriginals))
+            (progn
+              (and (buffer-modified-p buffer)
+                   (or (not TeX-save-query)
+                       (y-or-n-p (concat "Save file "
+                                         (buffer-file-name buffer)
+                                         "? ")))
+                   (save-excursion (set-buffer buffer) (save-buffer)))))))
+    (dolist (eo existingoriginals)
+      (if (file-newer-than-file-p eo derived)
+          (setq found t)))
     found))
 
 (defcustom TeX-save-query t
@@ -407,34 +409,38 @@ in `TeX-check-path'."
 
 (defun TeX-command-query (name)
   "Query the user for what TeX command to use."
-  (let* ((default (cond ((if (string-equal name TeX-region)
-			     (TeX-check-files (concat name "." (TeX-output-extension))
-					      (list name)
-					      TeX-file-extensions)
-			   (TeX-save-document (TeX-master-file)))
-			 TeX-command-default)
-			((and (memq major-mode '(doctex-mode latex-mode))
-			      (TeX-check-files (concat name ".bbl")
-					       (mapcar 'car
-						       (LaTeX-bibliography-list))
-					       BibTeX-file-extensions))
-			 ;; We should check for bst files here as well.
-			 TeX-command-BibTeX)
-			((TeX-process-get-variable name
-						   'TeX-command-next
-						   TeX-command-Show))
-			(TeX-command-Show)))
-	 (completion-ignore-case t)
-	 (answer (or TeX-command-force
-		     (completing-read
-		      (concat "Command: (default " default ") ")
-		      (TeX-mode-specific-command-list major-mode) nil t
-		      nil 'TeX-command-history))))
-    ;; If the answer "latex" it will not be expanded to "LaTeX"
+  (let* ((default
+	   (cond ((if (string-equal name TeX-region)
+		      (TeX-check-files (concat name "." (TeX-output-extension))
+				       (list name)
+				       TeX-file-extensions)
+		    (TeX-save-document (TeX-master-file)))
+		  TeX-command-default)
+		 ((and (memq major-mode '(doctex-mode latex-mode))
+		       ;; Want to know if bib file is newer than .bbl
+		       ;; We don't care whether the bib files are open in emacs
+		       (TeX-check-files (concat name ".bbl")
+					(mapcar 'car
+						(LaTeX-bibliography-list))
+					(append BibTeX-file-extensions
+						TeX-Biber-file-extensions)))
+		  ;; We should check for bst files here as well.
+		  (if LaTeX-using-Biber TeX-command-Biber TeX-command-BibTeX))
+		 ((TeX-process-get-variable name
+					    'TeX-command-next
+					    TeX-command-Show))
+		 (TeX-command-Show)))
+         (completion-ignore-case t)
+         (answer (or TeX-command-force
+                     (completing-read
+                      (concat "Command: (default " default ") ")
+                      (TeX-mode-specific-command-list major-mode) nil t
+                      nil 'TeX-command-history))))
+    ;; If the answer is "latex" it will not be expanded to "LaTeX"
     (setq answer (car-safe (TeX-assoc answer TeX-command-list)))
     (if (and answer
-	     (not (string-equal answer "")))
-	answer
+             (not (string-equal answer "")))
+        answer
       default)))
 
 (defvar TeX-command-next nil
@@ -558,6 +564,7 @@ Return the new process."
     (setq-default TeX-command-buffer command-buff)
     (get-buffer-create buffer)
     (set-buffer buffer)
+    (buffer-disable-undo)
     (erase-buffer)
     (set (make-local-variable 'line-number-display-limit) 0)
     (setq TeX-output-extension nil)
@@ -672,6 +679,14 @@ run of `TeX-run-TeX', use
 	process
       (TeX-synchronous-sentinel name file process))))
 
+(defun TeX-run-Biber (name command file)
+  "Create a process for NAME using COMMAND to format FILE with Biber." 
+  (let ((process (TeX-run-command name command file)))
+    (setq TeX-sentinel-function 'TeX-Biber-sentinel)
+    (if TeX-process-asynchronous
+        process
+      (TeX-synchronous-sentinel name file process))))
+
 (defun TeX-run-compile (name command file)
   "Ignore first and third argument, start compile with second argument."
   (compile command))
@@ -684,7 +699,8 @@ run of `TeX-run-TeX', use
 	(redraw-display))))
 
 (defun TeX-run-discard (name command file)
-  "Start process with second argument, discarding its output."
+  "Start COMMAND as process, discarding its output.
+NAME and FILE are ignored."
   (let ((default-directory (TeX-master-directory)))
     (call-process TeX-shell
 		  nil 0 nil
@@ -774,6 +790,14 @@ Parameters NAME and FILE are ignored."
   (let ((fun (car (read-from-string command))))
     (if (functionp fun) (funcall fun) (eval fun))))
 
+(defun TeX-run-discard-or-function (name command file)
+  "Start COMMAND as process or execute it as a Lisp function.
+If run as a process, the output is discarded.  COMMAND is
+expected to be a string.  NAME and FILE are ignored."
+  (if (functionp (car (read-from-string command)))
+      (TeX-run-function name command file)
+    (TeX-run-discard name command file)))
+
 (defun TeX-run-ispell-on-document (command ignored name)
   "Run ispell on all open files belonging to the current document.
 This function is *obsolete* and only here for compatibility
@@ -786,7 +810,7 @@ reasons.  Use `TeX-run-function' instead."
 
 (defun TeX-synchronous-sentinel (name file result)
   "Process TeX command output buffer after the process dies."
-  (let* ((buffer (TeX-process-buffer file)))
+  (let ((buffer (TeX-process-buffer (file-name-nondirectory file))))
     (save-excursion
       (set-buffer buffer)
 
@@ -809,8 +833,10 @@ reasons.  Use `TeX-run-function' instead."
   "Process TeX command output buffer after the process dies."
   ;; Set `TeX-transient-master' here because `preview-parse-messages'
   ;; may open files and thereby trigger master file questions which we
-  ;; don't want and need because we already know the master.
-  (let* ((TeX-transient-master (TeX-active-master))
+  ;; don't want and need because we already know the master.  Use
+  ;; `TeX-master-file' instead of `TeX-active-master' to determine the
+  ;; master because the region file should never be the master.
+  (let* ((TeX-transient-master (TeX-master-file))
 	 (buffer (process-buffer process))
 	 (name (process-name process)))
     (cond ((null (buffer-name buffer))	; buffer killed
@@ -869,11 +895,11 @@ NAME is the name of the process.")
 (defun TeX-current-pages ()
   "Return string indicating the number of pages formatted."
   (cond ((null TeX-current-page)
-	 "some pages.")
+	 "some pages")
 	((string-match "[^0-9]1[^0-9]" TeX-current-page)
-	 (concat TeX-current-page " page."))
+	 (concat TeX-current-page " page"))
 	(t
-	 (concat TeX-current-page " pages."))))
+	 (concat TeX-current-page " pages"))))
 
 (defun TeX-TeX-sentinel-check (process name)
   "Cleanup TeX output buffer after running TeX.
@@ -927,38 +953,63 @@ Warnings can be indicated by LaTeX or packages."
   "Cleanup TeX output buffer after running LaTeX."
   (cond ((TeX-TeX-sentinel-check process name))
 	((and (save-excursion
-		(or
-		 (re-search-forward "^LaTeX Warning: Citation" nil t)
-		 (re-search-forward "^Package natbib Warning: Citation" nil t)))
+		(re-search-forward
+		 "^Package biblatex Warning: Please (re)run Biber on the file"
+		 nil t))
 	      (with-current-buffer TeX-command-buffer
 		(and (LaTeX-bibliography-list)
 		     (TeX-check-files (TeX-master-file "bbl")
 				      (TeX-style-list)
 				      (append TeX-file-extensions
-					      BibTeX-file-extensions)))))
-	 (message (concat "You should run BibTeX to get citations right, "
-			  (TeX-current-pages)))
+					      BibTeX-file-extensions
+					      TeX-Biber-file-extensions)))))
+	 (message "%s%s" "You should run Biber to get citations right, "
+		  (TeX-current-pages))
+	 (setq TeX-command-next (with-current-buffer TeX-command-buffer
+				  TeX-command-Biber)))
+	((and (save-excursion
+		(re-search-forward
+		 "^\\(?:LaTeX\\|Package natbib\\) Warning: Citation" nil t))
+	      (with-current-buffer TeX-command-buffer
+		(and (LaTeX-bibliography-list)
+		     (TeX-check-files (TeX-master-file "bbl")
+				      (TeX-style-list)
+				      (append TeX-file-extensions
+					      BibTeX-file-extensions
+					      TeX-Biber-file-extensions)))))
+	 (message "%s%s" "You should run BibTeX to get citations right, "
+		  (TeX-current-pages))
 	 (setq TeX-command-next (with-current-buffer TeX-command-buffer
 				  TeX-command-BibTeX)))
-	((or
-	  (re-search-forward "^LaTeX Warning: Label(s)" nil t)
-	  (re-search-forward "^Package natbib Warning: Citation(s)" nil t))
-	 (message (concat "You should run LaTeX again "
-			  "to get references right, "
-			  (TeX-current-pages)))
+  ((re-search-forward "Package biblatex Warning: Please rerun LaTeX" nil t)
+	 (message "%s%s" "You should run LaTeX again, " (TeX-current-pages))
+	 (setq TeX-command-next TeX-command-default))
+	((re-search-forward "^(biblatex)\\W+Page breaks have changed" nil t)
+	 (message "%s%s" "You should run LaTeX again - page breaks have changed, "
+		  (TeX-current-pages))
+	 (setq TeX-command-next TeX-command-default))
+	((re-search-forward "^\\(?:LaTeX Warning: Label(s)\\|\
+Package natbib Warning: Citation(s)\\)" nil t)
+	 (message "%s%s" "You should run LaTeX again to get references right, "
+		  (TeX-current-pages))
 	 (setq TeX-command-next TeX-command-default))
 	((re-search-forward "^LaTeX Warning: Reference" nil t)
-	 (message (concat name ": there were unresolved references, "
-			  (TeX-current-pages)))
+	 (message "%s%s%s" name ": there were unresolved references, "
+		  (TeX-current-pages))
 	 (setq TeX-command-next TeX-command-Show))
-	((or
-	  (re-search-forward "^LaTeX Warning: Citation" nil t)
-	  (re-search-forward "^Package natbib Warning:.*undefined citations" nil t))
-	 (message (concat name ": there were unresolved citations, "
-			  (TeX-current-pages)))
+	((re-search-forward "^\\(?:LaTeX Warning: Citation\\|\
+Package natbib Warning:.*undefined citations\\)" nil t)
+	 (message "%s%s%s" name ": there were unresolved citations, "
+		  (TeX-current-pages))
 	 (setq TeX-command-next TeX-command-Show))
+	((re-search-forward "Package longtable Warning: Table widths have \
+changed\\. Rerun LaTeX\\." nil t)
+	 (message
+	  "%s" "You should run LaTeX again to get table formatting right")
+	 (setq TeX-command-next TeX-command-default))
 	((re-search-forward
-	  "^\\(\\*\\* \\)?J?I?p?\\(La\\|Sli\\)TeX\\(2e\\)? \\(Version\\|ver\\.\\|<[0-9/]*>\\)" nil t)
+	  "^\\(\\*\\* \\)?J?I?p?\\(La\\|Sli\\)TeX\\(2e\\)? \
+\\(Version\\|ver\\.\\|<[0-9/]*>\\)" nil t)
 	 (let* ((warnings (and TeX-debug-warnings
 			       (TeX-LaTeX-sentinel-has-warnings)))
 		(bad-boxes (and TeX-debug-bad-boxes
@@ -969,12 +1020,11 @@ Warnings can be indicated by LaTeX or packages."
 				    (when (and warnings bad-boxes) " and ")
 				    (when bad-boxes "bad boxes")
 				    ")"))))
-	   (message (concat name ": successfully formatted "
-			    (TeX-current-pages) add-info)))
+	   (message "%s" (concat name ": successfully formatted "
+				 (TeX-current-pages) add-info)))
 	 (setq TeX-command-next TeX-command-Show))
 	(t
-	 (message (concat name ": problems after "
-			  (TeX-current-pages)))
+	 (message "%s%s%s" name ": problems after " (TeX-current-pages))
 	 (setq TeX-command-next TeX-command-default))))
 
 ;; should go into latex.el? --pg
@@ -995,8 +1045,32 @@ Warnings can be indicated by LaTeX or packages."
 	      "\\<TeX-mode-map>\\[TeX-recenter-output-buffer]")))
    (t
     (message (concat "BibTeX finished successfully. "
-		     "Run LaTeX again to get citations right."))))
-  (setq TeX-command-next TeX-command-default))
+		     "Run LaTeX again to get citations right."))
+  (setq TeX-command-next TeX-command-default))))
+
+(defun TeX-Biber-sentinel (process name)
+  "Cleanup TeX output buffer after running Biber."
+  (goto-char (point-max))
+  (cond
+   ((re-search-backward (concat
+                         "^INFO - \\(WARNINGS\\|ERRORS\\): \\([0-9]+\\)") nil t)
+    (message (concat "Biber finished with %s %s. "
+                     "Type `%s' to display output.")
+             (match-string 2) (downcase (match-string 1))
+             (substitute-command-keys
+              "\\<TeX-mode-map>\\[TeX-recenter-output-buffer]"))
+    (setq TeX-command-next TeX-command-default))
+   ((re-search-backward (concat
+                         "^FATAL") nil t)
+    (message (concat "Biber had a fatal error and did not finish! "
+                     "Type `%s' to display output.")
+             (substitute-command-keys
+              "\\<TeX-mode-map>\\[TeX-recenter-output-buffer]"))
+    (setq TeX-command-next TeX-command-Biber))
+   (t
+    (message (concat "Biber finished successfully. "
+                     "Run LaTeX again to get citations right."))
+    (setq TeX-command-next TeX-command-default))))
 
 ;;; Process Control
 
@@ -1101,11 +1175,11 @@ command."
 	;; Remove line breaks at column 79
 	(while (> (point) pt)
 	  (end-of-line 0)
-	  (when (and (= (current-column) 79)
-		     ;; Heuristic: Don't delete the linebreak if there
-		     ;; is an empty line after the current one or
-		     ;; point is located after a period.
-		     (not (eq (char-after (1+ (point))) ?\n))
+	  (when (and (= (- (point) (line-beginning-position)) 79)
+		     ;; Heuristic: Don't delete the linebreak if the
+		     ;; next line is empty or starts with an opening
+		     ;; parenthesis or if point is located after a period.
+		     (not (memq (char-after (1+ (point))) '(?\n ?\()))
 		     (not (eq (char-before) ?.)))
 	    (delete-char 1)))
 	(goto-char (marker-position (process-mark process)))
@@ -1401,23 +1475,32 @@ You might want to examine and modify the free variables `file',
 
 (defun TeX-parse-error (old)
   "Goto next error.  Pop to OLD buffer if no more errors are found."
+  (let ((regexp
+	 (concat
+	  ;; TeX error
+	  "^\\(!\\|\\(.*?\\):[0-9]+:\\) \\|"
+	  ;; New file
+	  "(\\(\"[^\"]*?\"\\|/*\
+\\(?:\\.+[^()\r\n{} \\/]*\\|[^()\r\n{} .\\/]+\
+\\(?: [^()\r\n{} .\\/]+\\)*\\(?:\\.[-0-9a-zA-Z_.]*\\)?\\)\
+\\(?:[\\/]+\\(?:\\.+[^()\r\n{} \\/]*\\|[^()\r\n{} .\\/]+\
+\\(?: [^()\r\n{} .\\/]+\\)*\\(?:\\.[-0-9a-zA-Z_.]*\\)?\\)?\\)*\\)\
+)*\\(?: \\|\r?$\\)\\|"
+	  ;; End of file
+	  "\\()\\))*\\|"
+	  ;; Hook to change line numbers
+	  " !\\(?:offset(\\([---0-9]+\\))\\|"
+	  ;; Hook to change file name
+	  "name(\\([^)]+\\))\\)\\|"
+	  ;; LaTeX bad box
+	  "^\\(\\(?:Overfull\\|Underfull\\|Tight\\|Loose\\)\
+ \\\\.*?[0-9]+--[0-9]+\\)\\|"
+	  ;; LaTeX warning
+	  "^\\(LaTeX [A-Za-z]*\\|Package [A-Za-z]+ \\)Warning:.*")))
     (while
 	(cond
-	 ((null (re-search-forward
-		 "\
-^\\(!\\|\\(.*?\\):[0-9]+:\\) \\|\
-\(\\(/*\
-\\(?:\\.+[^()\r\n{} /]*\\|[^()\r\n{} ./]+\
-\\(?: [^()\r\n{} ./]+\\)*\\(?:\\.[-0-9a-zA-Z_.]*\\)?\\)\
-\\(?:/+\\(?:\\.+[^()\r\n{} /]*\\|[^()\r\n{} ./]+\
-\\(?: [^()\r\n{} ./]+\\)*\\(?:\\.[-0-9a-zA-Z_.]*\\)?\\)?\\)*\\)\
-)*\\(?: \\|\r?$\\)\\|\
-\\()\\))*\\|\
- !\\(?:offset(\\([---0-9]+\\))\\|\
-name(\\([^)]+\\))\\)\\|\
-^\\(\\(?:Overfull\\|Underfull\\|Tight\\|Loose\\)\
- \\\\.*?[0-9]+--[0-9]+\\)\\|\
-^\\(LaTeX [A-Za-z]*\\|Package [A-Za-z]+ \\)Warning:.*" nil t))
+	 ((null
+	   (re-search-forward regexp nil t))
 	  ;; No more errors.
 	  (message "No more errors.")
 	  (beep)
@@ -1435,7 +1518,7 @@ name(\\([^)]+\\))\\)\\|\
 	      t
 	    (TeX-error)
 	    nil))
-	 ;; LaTeX badbox
+	 ;; LaTeX bad box
 	 ((match-beginning 7)
 	  (if TeX-debug-bad-boxes
 	      (progn
@@ -1455,14 +1538,21 @@ name(\\([^)]+\\))\\)\\|\
 
 	 ;; New file -- Push on stack
 	 ((match-beginning 3)
-	  (push (TeX-match-buffer 3) TeX-error-file)
-	  (push nil TeX-error-offset)
-	  (goto-char (match-end 3))
+	  (let ((file (TeX-match-buffer 3))
+		(end (match-end 3)))
+	    ;; Strip quotation marks and remove newlines if necessary
+	    (when (or (eq (string-to-char file) ?\")
+		      (string-match "\n" file))
+	      (setq file
+		    (mapconcat 'identity (split-string file "[\"\n]+") "")))
+	    (push file TeX-error-file)
+	    (push nil TeX-error-offset)
+	    (goto-char end))
 	  t)
 	 
 	 ;; End of file -- Pop from stack
 	 ((match-beginning 4)
-	  (when (> (length TeX-error-file) 1)
+	  (when (> (length TeX-error-file) 0)
 	    (pop TeX-error-file)
 	    (pop TeX-error-offset))
 	  (goto-char (match-end 4))
@@ -1471,14 +1561,14 @@ name(\\([^)]+\\))\\)\\|\
 	 ;; Hook to change line numbers
 	 ((match-beginning 5)
 	  (setq TeX-error-offset
-		(list (string-to-int (TeX-match-buffer 5))))
+		(list (string-to-number (TeX-match-buffer 5))))
 	  t)
 	 
 	 ;; Hook to change file name
 	 ((match-beginning 6)
 	  (setq TeX-error-file
 		(list (TeX-match-buffer 6)))
-	  t))))
+	  t)))))
 
 (defun TeX-error ()
   "Display an error."
@@ -1490,11 +1580,22 @@ name(\\([^)]+\\))\\)\\|\
 
 	 ;; And the context for the help window.
 	 (context-start (point))
+	 context-available
 
 	 ;; And the line number to position the cursor.
-	 (line (if (re-search-forward "l\\.\\([0-9]+\\)" nil t)
-		   (string-to-int (TeX-match-buffer 1))
-		 1))
+	 (line (cond
+		;; regular style
+		((re-search-forward "l\\.\\([0-9]+\\)" nil t)
+		 (setq context-available t)
+		 (string-to-number (TeX-match-buffer 1)))
+		;; file:line:error style
+		((save-excursion
+		   (re-search-backward ":\\([0-9]+\\): "
+				       (line-beginning-position) t))
+		 (string-to-number (TeX-match-buffer 1)))
+		;; nothing found
+		(t 1)))
+
 	 ;; And a string of the context to search for.
 	 (string (progn
 		   (beginning-of-line)
@@ -1502,10 +1603,14 @@ name(\\([^)]+\\))\\)\\|\
 		   (TeX-match-buffer 1)))
 
 	 ;; And we have now found to the end of the context.
-	 (context (buffer-substring context-start (progn
-						    (forward-line 1)
-						    (end-of-line)
-						    (point))))
+	 (context (if context-available
+		      (buffer-substring context-start (progn (forward-line 1)
+							     (end-of-line)
+							     (point)))
+		    ;; There is no real context available, so we
+		    ;; simply show the line with the error message.
+		    (buffer-substring (1- (line-beginning-position))
+				      context-start)))
 	 ;; We may use these in another buffer.
 	 (offset (or (car TeX-error-offset) 0))
 	 (file (car TeX-error-file)))
@@ -1548,7 +1653,7 @@ name(\\([^)]+\\))\\)\\|\
   (let* ((error (concat "** " string))
 
 	 ;; bad-box is nil if this is a "LaTeX Warning"
-	 (bad-box (string-match "\\\\[vb]ox.*[0-9]*--[0-9]*" string))
+	 (bad-box (string-match "\\\\[vh]box.*[0-9]*--[0-9]*" string))
 	 ;; line-string: match 1 is beginning line, match 2 is end line
 	 (line-string (if bad-box " \\([0-9]*\\)--\\([0-9]*\\)"
 			"on input line \\([0-9]*\\)\\."))
@@ -1558,8 +1663,8 @@ name(\\([^)]+\\))\\)\\|\
 
 	 ;; Get error-line (warning)
 	 (line (when (re-search-backward line-string nil t)
-		 (string-to-int (TeX-match-buffer 1))))
-	 (line-end (if bad-box (string-to-int (TeX-match-buffer 2))
+		 (string-to-number (TeX-match-buffer 1))))
+	 (line-end (if bad-box (string-to-number (TeX-match-buffer 2))
 		     line))
 
 	 ;; Find the context
@@ -1590,6 +1695,9 @@ name(\\([^)]+\\))\\)\\|\
     (goto-char error-point)
     (setq TeX-error-point (point))
 
+    (unless file
+      (error "Could not determine file for warning"))
+
     ;; Go back to TeX-buffer
     (let ((runbuf (current-buffer))
 	  (master (with-current-buffer
@@ -1647,32 +1755,30 @@ name(\\([^)]+\\))\\)\\|\
 	    "\n\n--- TeX said ---"
 	    output
 	    "\n--- HELP ---\n"
-	    (save-excursion
-	      (if (and (string= (cdr (nth TeX-error-pointer
-					  TeX-error-description-list))
-				"No help available")
-		       (let* ((log-buffer (find-buffer-visiting log-file)))
-			 (if log-buffer
-			     (progn
-			       (set-buffer log-buffer)
-			       (revert-buffer t t))
-			   (setq log-buffer
-				 (find-file-noselect log-file))
-			   (set-buffer log-buffer))
-			 (auto-save-mode nil)
-			 (setq buffer-read-only t)
-			 (goto-line (point-min))
-			 (search-forward error nil t 1)))
-		  (progn
-		    (re-search-forward "^l\\.")
-		    (re-search-forward "^ [^\n]+$")
-		    (forward-char 1)
-		    (let ((start (point)))
+	    (let ((help (cdr (nth TeX-error-pointer
+				  TeX-error-description-list))))
+	      (save-excursion
+		(if (and (string= help "No help available")
+			 (let* ((log-buffer (find-buffer-visiting log-file)))
+			   (if log-buffer
+			       (progn
+				 (set-buffer log-buffer)
+				 (revert-buffer t t))
+			     (setq log-buffer
+				   (find-file-noselect log-file))
+			     (set-buffer log-buffer))
+			   (auto-save-mode nil)
+			   (setq buffer-read-only t)
+			   (goto-line (point-min))
+			   (search-forward error nil t 1))
+			 (re-search-forward "^l\\." nil t)
+			 (re-search-forward "^ [^\n]+$" nil t))
+		    (let ((start (1+ (point))))
+		      (forward-char 1)
 		      (re-search-forward "^$")
 		      (concat "From the .log file...\n\n"
-			      (buffer-substring start (point)))))
-		(cdr (nth TeX-error-pointer
-			  TeX-error-description-list)))))
+			      (buffer-substring start (point))))
+		  help))))
     (goto-char (point-min))
     (TeX-pop-to-buffer old-buffer nil t)))
 
